//
// svg image display demo program for the Fast Light Tool Kit (FLTK).
//
// Copyright (c) 2016 by Albrecht Schlosser
//
// This library is free software. Distribution and use rights are outlined in
// the file "COPYING" which should have been included with this file.  If this
// file is missing or damaged, see the license at:
//
//     http://www.fltk.org/COPYING.php
//
// Please report all bugs and problems on the following page:
//
//     http://www.fltk.org/str.php

//
// Build instructions:
//
// Download the two nanosvg header files nanosvg.h and nanosvgrast.h
// (see links below) to the same folder as this file (svg.cxx).
//
// Compile with: fltk-config --compile svg.cxx
//
// Usage: ./svg image.svg
//
// Resize the window to see how the image scales...
//
// Note: this is an early version of a demo program. Debug statements are
// included (mostly commented out). This is not a fully functional program
// and may need modifications for some images (particularly the background
// color).
//
// Enjoy!
//
// Version 0.9.0: file selection at command line; scales the image to fit.

#include <FL/Fl.H>
#include <FL/Fl_Window.H>
#include <FL/Fl_Box.H>
#include <FL/Fl_RGB_Image.H>
#include <FL/fl_draw.H>

class Fl_SVG_Box;			// displays scaled SVG image

/*
  The entire nanosvg library code is included by defining the two macros
  NANOSVG*_IMPLEMENTATION before including the headers.

  Source: https://github.com/memononen/nanosvg.git

  Direct download links:

  https://raw.githubusercontent.com/memononen/nanosvg/master/src/nanosvg.h
  https://raw.githubusercontent.com/memononen/nanosvg/master/src/nanosvgrast.h
*/

#include <stdio.h>
#include <string.h>
#include <math.h>

#define NANOSVG_ALL_COLOR_KEYWORDS	// Include full list of color keywords.
#define NANOSVG_IMPLEMENTATION		// Expands implementation
#include "nanosvg.h"

#define NANOSVGRAST_IMPLEMENTATION	// Expands implementation
#include "nanosvgrast.h"

// Globals (temporary)

struct NSVGrasterizer* rast = 0;
NSVGimage* svg_image = 0;
int iw = 0, ih = 0;
unsigned char* img = 0;
Fl_SVG_Box *box = 0;
Fl_RGB_Image *rgb = 0;

// class Fl_SVG_Box: resizable image display widget

class Fl_SVG_Box : public Fl_Box {
public:
  Fl_SVG_Box (int X, int Y, int W, int H)
  : Fl_Box (X, Y, W, H) {}
  virtual void resize(int X, int Y, int W, int H)
  {
    Fl_Widget::resize(X,Y,W,H);
    if (img) free((void*)img);

    float f1 = (float(W) - 5.0) / iw;
    float f2 = (float(H) - 5.0) / ih;
    float f = (f1 < f2) ? f1 : f2;

    int w1 = int(iw*f);
    int h1 = int(ih*f);
#if (0)
    printf("Factor(s): %f, %f, %f, size: (%d,%d)\n",
	    f1, f2, f, w1, h1);
#endif

    // Allocate memory for image
    img = (unsigned char*)malloc(w1*h1*4);

    // Rasterize
    nsvgRasterize(rast, svg_image, 0, 0, f, img, w1, h1, w1*4);

    if (rgb) delete rgb;
    rgb = new Fl_RGB_Image(img, w1, h1, 4, 0);
    if (rgb->fail()) {
      printf("RGB image could not be loaded, error = %d\n", rgb->fail());
      exit(2);
    }
#if (0)
    printf("RGB: %d x %d\n", rgb->w(), rgb->h());
#endif
    image(rgb);
  }
};

int main(int argc, char **argv) {

  Fl_Color bg = FL_WHITE;			// default background color
  bg = fl_rgb_color(0xff, 0xdd, 0xff);		// bright pink bg color

  Fl_Window *window = new Fl_Window(620,620);
  window->color(bg);

  box = new Fl_SVG_Box(5,5,610,610);
  box->box(FL_FLAT_BOX);
  box->color(bg);
  box->align(FL_ALIGN_CENTER|FL_ALIGN_INSIDE);
  window->end();

  const char *filename = "tiger.svg"; // default file name
  if (argc > 1)
    filename = argv[1];

  // Parse svg file

  svg_image = nsvgParseFromFile(filename, "px", 96);

  if (svg_image) {
    iw = int(svg_image->width);
    ih = int(svg_image->height);
    printf("size: %f x %f\n", svg_image->width, svg_image->height);
  } else {
    printf("Image not found. Exiting. :-(");
    exit(1);
  }

  // Create rasterizer (can be used to render multiple images).
  rast = nsvgCreateRasterizer();

  // Load image: using resize()
  box->resize(box->x(), box->y(), box->w(), box->h());

  window->resizable(box);
  window->show(1 /* argc */, argv);
  return Fl::run();
}
